package com.ankaibei.workFlow.webadmin.ankaibei.api;

import com.ankaibei.workFlow.webadmin.ankaibei.entity.ProjectInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.CodeEnum;
import com.ankaibei.workFlow.webadmin.ankaibei.service.ProjectInfoService;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ProjectInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ResMsg;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lwq
 * @version 1.0
 * @description:
 * @date 2022/9/14 11:11
 */
@Service
public class ProjectInfoServiceApi implements ProjectInfoApi {

    @Autowired
    private ProjectInfoService projectInfoService;

    @Override
    public ResMsg findProjectInfo(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.findProjectInfo(projectInfoVo.getPageIndex(), projectInfoVo.getPageSize(), projectInfoEntity);
    }

    @Override
    public ResMsg findProjectInfoAll(String projectName) {
        return projectInfoService.findProjectInfoAll(projectName);
    }

    @Override
    public ResMsg findProjectInfoAtFlowStatus(Integer flowTaskStatus) {
        return projectInfoService.findProjectInfoAtFlowStatus(flowTaskStatus);
    }

    @Override
    public ResMsg findProjectParticulars(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.findProjectParticulars(projectInfoEntity);
    }

    @Override
    public ResMsg addProject(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.addProject(projectInfoEntity);
    }

    @Override
    public ResMsg submitProjectBySave(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.submitProjectBySave(projectInfoEntity);
    }

    @Override
    public ResMsg submitProjectBatch(List<ProjectInfoVo> projectInfoVos) {
        if (projectInfoVos.size() == 0) {
            return new ResMsg(CodeEnum.PARAM_IS_NULL);
        }
        ArrayList<ProjectInfoEntity> list = new ArrayList<>();
        for (ProjectInfoVo vo : projectInfoVos) {
            ProjectInfoEntity infoEntity = new ProjectInfoEntity();
            BeanUtils.copyProperties(vo, infoEntity);
            list.add(infoEntity);
        }
        return projectInfoService.submitProjectBatch(list);
    }

    @Override
    public ResMsg updateProject(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.updateProject(projectInfoEntity);
    }

    @Override
    public ResMsg submitProjectByUpdate(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.submitProjectByUpdate(projectInfoEntity);
    }

    @Override
    public ResMsg findPrincipalProject(String userId) {

        return projectInfoService.findPrincipalProject(userId);
    }

    @Override
    public ResMsg findAccesoryInfo(ProjectInfoVo projectInfoVo) {
        ProjectInfoEntity projectInfoEntity = this.copyProperties(projectInfoVo, ProjectInfoEntity.class);
        return projectInfoService.findAccesoryInfo(projectInfoEntity);
    }

    @Override
    public ResMsg findProjectNumber() {

        return projectInfoService.findProjectNumber();
    }

    @Override
    public ResMsg substepOfProjectNum() {
        return projectInfoService.substepOfProjectNum();
    }

    @Override
    public ResMsg findLastMonthProjectNumber() {
        return projectInfoService.findLastMonthProjectNumber();
    }

    @Override
    public ResMsg substepOfProjectNumAtLastMonth() {
        return projectInfoService.substepOfProjectNumAtLastMonth();
    }

    @Override
    public ResMsg deleteProjects(List<ProjectInfoVo> projectInfoVos) {
        return projectInfoService.deleteProjects(projectInfoVos);
    }
}
