package com.ankaibei.workFlow.webadmin.ankaibei.controller;

import com.ankaibei.workFlow.common.core.object.TokenData;
import com.ankaibei.workFlow.webadmin.ankaibei.api.ManHourInfoApi;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.CodeEnum;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ManHourInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ResMsg;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * @author lwq
 * @version 1.0
 * @description: 工时管理
 * @date 2022/9/21 15:28
 */
@RestController
@RequestMapping("/manHour")
public class ManHourController {

    @Resource
    private ManHourInfoApi manHourInfoApi;

    /**
     * 查询所有工时列表
     */
    @RequestMapping("/findManHourAll")
    public ResMsg findManHourAll(@RequestBody ManHourInfoVo manHourInfoVo) {
        return manHourInfoApi.findManHourAll(manHourInfoVo);
    }

    /**
     * 工时详情
     */
    @RequestMapping("/findManHourByID")
    public ResMsg findManHourByID(@RequestBody ManHourInfoVo manHourInfoVo) {
        return manHourInfoApi.findManHourByID(manHourInfoVo);
    }

    /**
     * 新建工时
     */
    @RequestMapping("/addManHour")
    public ResMsg addManHour(@RequestBody ManHourInfoVo manHourInfoVo) {
        manHourInfoVo.setCreateUser(TokenData.takeFromRequest().getUserId());
        return manHourInfoApi.addManHour(manHourInfoVo);
    }

    /**
     * 保存后提交工时
     */
    @RequestMapping("/submitManHourBySave")
    public ResMsg submitManHourBySave(@RequestBody ManHourInfoVo manHourInfoVo) {
        manHourInfoVo.setCreateUser(TokenData.takeFromRequest().getUserId());
        return manHourInfoApi.submitManHourBySave(manHourInfoVo);
    }

    /**
     * 批量提交工时
     */
    @RequestMapping("/submitManHourBatch")
    public ResMsg submitManHourBatch(@RequestBody List<ManHourInfoVo> manHourInfoVos) {
        return manHourInfoApi.submitManHourBatch(manHourInfoVos);
    }

    /**
     * 编辑工时
     */
    @RequestMapping("/editManHour")
    public ResMsg editManHour(@RequestBody ManHourInfoVo manHourInfoVo) {
        manHourInfoVo.setCreateUser(TokenData.takeFromRequest().getUserId());
        return manHourInfoApi.editManHour(manHourInfoVo);
    }

    /**
     * 编辑工时同时提交
     */
    @RequestMapping("/submitManHourByeEdit")
    public ResMsg submitManHourByeEdit(@RequestBody ManHourInfoVo manHourInfoVo) {
        manHourInfoVo.setCreateUser(TokenData.takeFromRequest().getUserId());
        return manHourInfoApi.submitManHourByeEdit(manHourInfoVo);
    }

    /**
     * 删除
     *
     * @param manHourInfoVos
     * @return ResMsg
     */
    @RequestMapping("/deleteManHour")
    public ResMsg deleteManHour(@RequestBody List<ManHourInfoVo> manHourInfoVos) {
        return manHourInfoApi.deleteManHour(manHourInfoVos);
    }


}
