package com.ankaibei.workFlow.webadmin.ankaibei.controller;

import com.ankaibei.workFlow.common.core.object.TokenData;
import com.ankaibei.workFlow.common.flow.constant.FlowTaskStatus;
import com.ankaibei.workFlow.webadmin.ankaibei.api.TaskInfoApi;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.TaskStatus;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ResMsg;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.TaskInfoVo;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lwq
 * @version 1.0
 * @description: 任务列表
 * @date 2022/9/14 17:14
 */
@RestController
@RequestMapping("/task")
public class TaskInfoController {

    @Resource
    private TaskInfoApi taskInfoApi;

    /**
     * 查询任务
     *
     * @param taskInfoVo
     * @return ResMsg
     */
    @RequestMapping("/findTaskInfo")
    public ResMsg findTaskInfo(@RequestBody TaskInfoVo taskInfoVo) {

        return taskInfoApi.findTaskInfo(taskInfoVo);
    }

    /**
     * 查询任务详情
     *
     * @param taskInfoVo
     * @return ResMsg
     */
    @RequestMapping("/findTaskById")
    public ResMsg findTaskById(@RequestBody TaskInfoVo taskInfoVo) {

        return taskInfoApi.findTaskById(taskInfoVo);
    }

    /**
     * 新增任务
     *
     * @param taskInfoVo
     * @return ResMsg
     */
    @RequestMapping("/addTaskInfo")
    public ResMsg addTaskInfo(@RequestBody TaskInfoVo taskInfoVo) {
        taskInfoVo.setTaskCreator(TokenData.takeFromRequest().getUserId());
        return taskInfoApi.addTaskInfo(taskInfoVo);
    }

    /**
     * 保存后提交任务
     */
    @RequestMapping("/submitTaskBySave")
    public ResMsg submitTaskBySave(@RequestBody TaskInfoVo taskInfoVo) {
        taskInfoVo.setTaskCreator(TokenData.takeFromRequest().getUserId());
        return taskInfoApi.submitTaskBySave(taskInfoVo);
    }

    /**
     * 批量提交
     */
    @RequestMapping("/submitTaskBatch")
    public ResMsg submitTaskBatch(@RequestBody List<TaskInfoVo> taskInfoVos) {
        return taskInfoApi.submitTaskBatch(taskInfoVos);
    }

    /**
     * 编辑任务--保存
     */
    @RequestMapping("/editTask")
    public ResMsg editTask(@RequestBody TaskInfoVo taskInfoVo) {
        taskInfoVo.setTaskCreator(TokenData.takeFromRequest().getUserId());
        return taskInfoApi.editTask(taskInfoVo);
    }

    /**
     * 编辑任务--提交
     */
    @RequestMapping("/submitTaskByEdit")
    public ResMsg submitTaskByEdit(@RequestBody TaskInfoVo taskInfoVo) {
        taskInfoVo.setTaskCreator(TokenData.takeFromRequest().getUserId());
        return taskInfoApi.submitTaskByEdit(taskInfoVo);
    }

    /**
     * 删除任务
     *
     * @param taskInfoVos
     * @return ResMsg
     */
    @RequestMapping("/deleteTask")
    public ResMsg deleteTask(@RequestBody List<TaskInfoVo> taskInfoVos) {
        return taskInfoApi.deleteTask(taskInfoVos);
    }


    /**
     * 完成任务
     *
     * @param taskInfoVo
     * @return ResMsg
     */
    @RequestMapping("/performTask")
    public ResMsg performTask(@RequestBody TaskInfoVo taskInfoVo) {

        return taskInfoApi.performTask(taskInfoVo);
    }

    /**
     * 查询登录用户已完成的任务（指定项目）
     *
     * @return ResMsg
     */
    @RequestMapping("/findTaskIsFinish")
    public ResMsg findTaskIsFinish(@RequestBody TaskInfoVo taskInfoVo) {
        String userId = TokenData.takeFromRequest().getUserId();
        taskInfoVo.setTaskParticipant(new ArrayList<Long>() {{
            add(Long.valueOf(userId));
        }});
        return taskInfoApi.findTaskIsFinish(taskInfoVo, Integer.parseInt(TaskStatus.FINISH_ED.getCode()));
    }


}
