package com.ankaibei.workFlow.webadmin.ankaibei.mapper;

import com.ankaibei.workFlow.webadmin.ankaibei.entity.TaskInfoEntity;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;

import javax.transaction.Transactional;
import java.util.List;
import java.util.Map;

/**
 * @author
 * @version 1.0
 * @description:
 * @date 2022/9/14 17:19
 */
public interface TaskInfoMapper extends JpaRepository<TaskInfoEntity, Integer>, JpaSpecificationExecutor<TaskInfoEntity> {

    /**
     * 分页查询
     *
     * @param targetTask
     * @param taskStatus
     * @param page
     * @return
     */
    @Query(value = " SELECT t.task_id taskId,t.target_task targetTask,t.task_priority taskPriority,t.task_participant taskParticipant," +
            " t.start_time startTime,t.end_time endTime,t.task_status taskStatus,t.task_creator taskCreator,t.task_index taskIndex," +
            " t.task_standard taskStandard,t.task_statements taskStatements,t.project_id projectId,t.process_instance_id processInstanceId," +
            " t.flow_task_status flowTaskStatus,t.process_definition_id processDefinitionId  from task_info t WHERE " +
            " if( ?1 !='', (t.task_participant LIKE %?1% or t.task_creator= ?1 ), 1=1) AND " +
            " if( ?2 !='', t.target_task = ?2 , 1=1) AND " +
            " if( ?3 !='', t.task_status = ?3 , 1=1) and is_delete='1' order by t.create_date desc ",
            countQuery = "select count(1)from task_info t WHERE " +
                    " if( ?1 !='', (t.task_participant LIKE %?1% or t.task_creator= ?1 ), 1=1) AND " +
                    " if( ?2 !='', t.target_task = ?2 , 1=1) AND " +
                    " if( ?3 !='', t.task_status = ?3 , 1=1) and is_delete='1'", nativeQuery = true)
    Page<Map<String, String>> findTaskInfo(String userId, String targetTask, String taskStatus, Pageable page);

    /**
     * 查询任务名称数量
     */
    @Query
    int countByTargetTaskEqualsAndIsDeleteEquals(String targetTask, String isDelete);


    /**
     * 查询任务名称数量(除本身)
     */
    @Query
    int countByTargetTaskEqualsAndIsDeleteEqualsAndTaskIdNot(String targetTask, String isDelete, Integer taskId);


    /**
     * 完成任务
     *
     * @param taskId
     * @return
     */
    @Transactional
    @Modifying
    @Query(value = "UPDATE task_info t SET t.task_status = 4 WHERE t.task_id = ?1 ", nativeQuery = true)
    int performTask(Integer taskId);

    /**
     * 任务延期
     *
     * @param taskId
     */
    @Transactional
    @Modifying
    @Query(value = "UPDATE task_info t SET t.task_status = 3 WHERE t.task_id = ?1 ", nativeQuery = true)
    void extensionTask(Integer taskId);

    /**
     * 修改任务的审核状态
     *
     * @param flowTaskStatus
     * @param taskId
     */
    @Transactional
    @Modifying
    @Query(value = "update task_info SET task_status = ?1, flow_task_status = ?2, ratify_user = ?3 WHERE task_id = ?2", nativeQuery = true)
    void updateTaskFlowStatus(String taskStatus, String flowTaskStatus, String taskRatifyUser, Integer taskId);

    /**
     * 删除任务
     *
     * @param taskId
     */
    @Transactional
    @Modifying
    @Query(value = "update task_info SET is_delete='-1' WHERE task_id = ?1", nativeQuery = true)
    void deleteTask(Integer taskId);


    /**
     * 查询登录用户已完成的任务（全部）
     */
    @Query
    List<TaskInfoEntity> findAllByTaskParticipantContainsAndTaskStatusEqualsAndIsDeleteEquals(String userId, String taskStatus, String isDelete);

    /**
     * 查询登录用户已完成的任务（指定项目）
     */
    @Query
    List<TaskInfoEntity> findAllByTaskParticipantContainsAndProjectIdEqualsAndTaskStatusEqualsAndIsDeleteEquals(String userId, String projectId, String taskStatus, String isDelete);
}
