package com.ankaibei.workFlow.webadmin.ankaibei.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ankaibei.workFlow.common.core.object.ResponseResult;
import com.ankaibei.workFlow.common.core.object.TokenData;
import com.ankaibei.workFlow.common.flow.constant.FlowConstant;
import com.ankaibei.workFlow.common.flow.constant.FlowTaskStatus;
import com.ankaibei.workFlow.common.flow.dto.FlowTaskCommentDto;
import com.ankaibei.workFlow.common.flow.model.FlowEntry;
import com.ankaibei.workFlow.common.flow.model.FlowEntryPublish;
import com.ankaibei.workFlow.common.flow.vo.TaskInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.CommunicationInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.CostInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.PurchaseInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.RepastExpenseEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.RestsInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.TrafficExpenseEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.TravelExpenseEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.UserInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.CodeEnum;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.CommunicationMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.CostInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.PurchaseInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.RepastExpenseMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.RestInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.TrafficExpenseMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.TravelExpenseMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.UserInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.service.ApplyForCostService;
import com.ankaibei.workFlow.webadmin.ankaibei.service.CostInfoService;
import com.ankaibei.workFlow.webadmin.ankaibei.utils.FlowTaskStart;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.CommunicationInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.PurchaseInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.RepastExpenseVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ResMsg;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.RestsInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.TrafficExpenseVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.TravelExpenseVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @author lwq
 * @version 1.0
 * @description:
 * @date 2022/9/26 16:18
 */
@Service
public class ApplyForCostServiceImpl implements ApplyForCostService {

    @Resource
    private FileUtil fileUtil;
    @Autowired
    private FlowTaskStart flowTaskStart;
    @Resource
    private RestInfoMapper restInfoMapper;
    @Resource
    private UserInfoMapper userInfoMapper;
    @Resource
    private CostInfoMapper costInfoMapper;
    @Resource
    private CostInfoService costInfoService;
    @Resource
    private PurchaseInfoMapper purchaseInfoMapper;
    @Resource
    private CommunicationMapper communicationMapper;
    @Resource
    private RepastExpenseMapper repastExpenseMapper;
    @Resource
    private TravelExpenseMapper travelExpenseMapper;
    @Resource
    private TrafficExpenseMapper trafficExpenseMapper;

    @Override
    public ResMsg addCommunication(CommunicationInfoVo communicationInfoVo) {
        CommunicationInfoEntity communicationInfoEntity = new CommunicationInfoEntity();
        BeanUtils.copyProperties(communicationInfoVo, communicationInfoEntity);
        communicationInfoEntity.setCreateTime(new Date());
        communicationInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        CommunicationInfoEntity save = communicationMapper.save(communicationInfoEntity);

        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("通讯费");
        UserInfoEntity userInfoEntity = findUser(communicationInfoVo.getCreateUser());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(communicationInfoVo.getProjectName());
        costInfoEntity.setPerformAmount(communicationInfoVo.getTotalMoney());
        costInfoEntity.setHappenDate(communicationInfoVo.getCreateDate());
        costInfoEntity.setPaymentDate(communicationInfoVo.getEndDate());
        costInfoEntity.setProposer(communicationInfoVo.getCreateUser());
        costInfoEntity.setApplyCostId(save.getCommunicationId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg communicationSubmitBySave(CommunicationInfoVo communicationInfoVo) {
        ResMsg addCom = addCommunication(communicationInfoVo);
        if (!addCom.getCode().equals("200")) {
            return addCom;
        }
        CommunicationInfoEntity save = (CommunicationInfoEntity) addCom.getData();
        /**
         * 通讯费用申请流程的启动
         */
        ResMsg resMsg = communicationFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        communicationMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg communicationSubmitBatch(List<CommunicationInfoVo> communicationInfoVos) {
        for (CommunicationInfoVo communicationInfoVo : communicationInfoVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(communicationInfoVo.getCommunicationId());
            costInfoEntity.setCostType("通讯费");
            //查询通讯费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //通讯费用申请流程的启动
            CommunicationInfoEntity save = (CommunicationInfoEntity) infoDetails.getData();
            ResMsg resMsg = communicationFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            communicationMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg communicationUpdate(CommunicationInfoVo communicationInfoVo) {
        //保存主体信息
        CommunicationInfoEntity communicationInfoEntity = new CommunicationInfoEntity();
        BeanUtils.copyProperties(communicationInfoVo, communicationInfoEntity);
        communicationInfoEntity.setCreateTime(new Date());
        communicationInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        CommunicationInfoEntity save = communicationMapper.save(communicationInfoEntity);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getCommunicationId(), "通讯费");
        UserInfoEntity userInfoEntity = findUser(communicationInfoVo.getCreateUser());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(communicationInfoVo.getProjectName());
        costInfo.setPerformAmount(communicationInfoVo.getTotalMoney());
        costInfo.setHappenDate(communicationInfoVo.getCreateDate());
        costInfo.setPaymentDate(communicationInfoVo.getEndDate());
        costInfo.setProposer(communicationInfoVo.getCreateUser());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg communicationSubmitByUpdate(CommunicationInfoVo communicationInfoVo) {
        //修改后提交
        ResMsg update = communicationUpdate(communicationInfoVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        CommunicationInfoEntity save = (CommunicationInfoEntity) update.getData();
        /**
         * 通讯费用申请流程的启动
         */
        ResMsg resMsg = communicationFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        communicationMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 调用通讯费用申请流程的启动
     * 获取开始节点之后的第一个任务节点的数据
     */
    public ResMsg communicationFlowStart(CommunicationInfoEntity communicationInfo) {
        //processDefinitionKey 通讯费用报销审批流程--流程标识
        String processDefinitionKey = "projectCommunicationAppF";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", communicationInfo.getProjectName());
        stringObjectHashMap.put("application_fee", communicationInfo.getApplicationFee());
        stringObjectHashMap.put("remark", communicationInfo.getRemark());
        stringObjectHashMap.put("communication_id", communicationInfo.getCommunicationId());
        stringObjectHashMap.put("cost_type", "通讯费");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(communicationInfo.getCommunicationId(), "通讯费");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg addPurchase(PurchaseInfoVo purchaseInfoVo) {
        //保存主体信息
        PurchaseInfoEntity purchaseInfoEntity = new PurchaseInfoEntity();
        BeanUtils.copyProperties(purchaseInfoVo, purchaseInfoEntity);
        purchaseInfoEntity.setPurchaseMessage(JSON.toJSONString(purchaseInfoVo.getPurchaseMessage()));
        purchaseInfoEntity.setCreateTime(new Date());
        PurchaseInfoEntity save = purchaseInfoMapper.save(purchaseInfoEntity);
        //保存到费用列表
        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("采购费");
        UserInfoEntity userInfoEntity = findUser(purchaseInfoVo.getCreateUser());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(purchaseInfoVo.getProjectName());
        costInfoEntity.setPerformAmount(purchaseInfoVo.getTotalMoney());
        costInfoEntity.setHappenDate(purchaseInfoVo.getPurchaseDate());
        costInfoEntity.setProposer(purchaseInfoVo.getCreateUser());
        costInfoEntity.setApplyCostId(save.getPurchaseId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setCreateDate(new Date());
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg purchaseSubmitBySave(PurchaseInfoVo purchaseInfoVo) {
        ResMsg addpurchase = addPurchase(purchaseInfoVo);
        if (!addpurchase.getCode().equals("200")) {
            return addpurchase;
        }
        PurchaseInfoEntity save = (PurchaseInfoEntity) addpurchase.getData();
        /**
         * 调用采购费用申请流程的启动
         */
        ResMsg resMsg = purchaseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        purchaseInfoMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg purchaseSubmitBatch(List<PurchaseInfoVo> purchaseInfoVos) {
        for (PurchaseInfoVo purchaseInfoVo : purchaseInfoVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(purchaseInfoVo.getPurchaseId());
            costInfoEntity.setCostType("采购费");
            //查询采购费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //采购费用申请流程的启动
            PurchaseInfoEntity save = (PurchaseInfoEntity) infoDetails.getData();
            ResMsg resMsg = purchaseFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            purchaseInfoMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg purchaseUpdate(PurchaseInfoVo purchaseInfoVo) {
        //保存主体信息
        PurchaseInfoEntity purchaseInfoEntity = new PurchaseInfoEntity();
        BeanUtils.copyProperties(purchaseInfoVo, purchaseInfoEntity);
        purchaseInfoEntity.setPurchaseMessage(JSON.toJSONString(purchaseInfoVo.getPurchaseMessage()));
        purchaseInfoEntity.setCreateTime(new Date());
        purchaseInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        PurchaseInfoEntity save = purchaseInfoMapper.save(purchaseInfoEntity);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getPurchaseId(), "采购费");
        UserInfoEntity userInfoEntity = findUser(purchaseInfoVo.getCreateUser());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(purchaseInfoVo.getProjectName());
        costInfo.setPerformAmount(purchaseInfoVo.getTotalMoney());
        costInfo.setHappenDate(purchaseInfoVo.getPurchaseDate());
        costInfo.setProposer(purchaseInfoVo.getCreateUser());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg purchaseSubmitByUpdate(PurchaseInfoVo purchaseInfoVo) {
        //修改后提交
        ResMsg update = purchaseUpdate(purchaseInfoVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        PurchaseInfoEntity save = (PurchaseInfoEntity) update.getData();
        //调用采购费用申请流程的启动
        ResMsg resMsg = purchaseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        purchaseInfoMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 调用采购费用申请流程的启动
     * 获取开始节点之后的第一个任务节点的数据
     */
    public ResMsg purchaseFlowStart(PurchaseInfoEntity purchaseInfoEntity) {
        //processDefinitionKey 项目采购费用报销审批流程--流程标识。
        String processDefinitionKey = "projectPurchaseApp";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", purchaseInfoEntity.getProjectName());
        stringObjectHashMap.put("purchase_message", purchaseInfoEntity.getPurchaseMessage().toString());
        stringObjectHashMap.put("purchase_date", purchaseInfoEntity.getPurchaseDate());
        stringObjectHashMap.put("create_user", purchaseInfoEntity.getCreateUser());
        stringObjectHashMap.put("purchase_id", purchaseInfoEntity.getPurchaseId());
        stringObjectHashMap.put("cost_type", "采购费");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(purchaseInfoEntity.getPurchaseId(), "采购费");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.updateCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg addRest(RestsInfoVo restsInfoVo) {
        //保存主体信息
        RestsInfoEntity restsInfoEntity = new RestsInfoEntity();
        BeanUtils.copyProperties(restsInfoVo, restsInfoEntity);
        restsInfoEntity.setRestsMessage(JSON.toJSONString(restsInfoVo.getRestsMessage()));
        restsInfoEntity.setCreateTime(new Date());
        RestsInfoEntity save = restInfoMapper.save(restsInfoEntity);
        //保存到费用列表
        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("其他费用");
        UserInfoEntity userInfoEntity = findUser(restsInfoVo.getCreateUser());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(restsInfoVo.getProjectName());
        costInfoEntity.setPerformAmount(restsInfoVo.getTotalMoney());
        costInfoEntity.setHappenDate(restsInfoVo.getCreateDate());
        costInfoEntity.setProposer(restsInfoVo.getCreateUser());
        costInfoEntity.setApplyCostId(save.getRestsId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg restSubmitBySave(RestsInfoVo restsInfoVo) {
        ResMsg addRest = addRest(restsInfoVo);
        if (!addRest.getCode().equals("200")) {
            return addRest;
        }
        RestsInfoEntity save = (RestsInfoEntity) addRest.getData();
        /**
         * 调用其他费用申请流程的启动
         */
        ResMsg resMsg = restFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        restInfoMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg restSubmitBatch(List<RestsInfoVo> restsInfoVos) {
        for (RestsInfoVo restsInfoVo : restsInfoVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(restsInfoVo.getRestsId());
            costInfoEntity.setCostType("其他费用");
            //查询其他费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //其他费用申请流程的启动
            RestsInfoEntity save = (RestsInfoEntity) infoDetails.getData();
            ResMsg resMsg = restFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            restInfoMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg restUpdate(RestsInfoVo restsInfoVo) {
        //保存主体信息
        RestsInfoEntity restsInfoEntity = new RestsInfoEntity();
        BeanUtils.copyProperties(restsInfoVo, restsInfoEntity);
        restsInfoEntity.setRestsMessage(JSON.toJSONString(restsInfoVo.getRestsMessage()));
        restsInfoEntity.setCreateTime(new Date());
        restsInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        RestsInfoEntity save = restInfoMapper.save(restsInfoEntity);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getRestsId(), "其他费用");
        UserInfoEntity userInfoEntity = findUser(restsInfoVo.getCreateUser());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(restsInfoVo.getProjectName());
        costInfo.setPerformAmount(restsInfoVo.getTotalMoney());
        costInfo.setHappenDate(restsInfoVo.getCreateDate());
        costInfo.setProposer(restsInfoVo.getCreateUser());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg restSubmitByUpdate(RestsInfoVo restsInfoVo) {
        //修改后提交
        ResMsg update = restUpdate(restsInfoVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        RestsInfoEntity save = (RestsInfoEntity) update.getData();
        //调用其他费用申请流程的启动
        ResMsg resMsg = restFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        restInfoMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 调用其他费用申请流程的启动
     * 获取开始节点之后的第一个任务节点的数据
     */
    public ResMsg restFlowStart(RestsInfoEntity restsInfoEntity) {
        //processDefinitionKey 项目其他费用报销审批流程--流程标识。
        String processDefinitionKey = "projeRestsInfoApp";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", restsInfoEntity.getProjectName());
        stringObjectHashMap.put("rests_message", restsInfoEntity.getRestsMessage().toString());
        stringObjectHashMap.put("total_money", 0.0);
        stringObjectHashMap.put("create_date", restsInfoEntity.getCreateDate());
        stringObjectHashMap.put("create_user", restsInfoEntity.getCreateUser());
        stringObjectHashMap.put("id", restsInfoEntity.getRestsId());
        stringObjectHashMap.put("cost_type", "其他费用");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(restsInfoEntity.getRestsId(), "其他费用");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg addTraffic(TrafficExpenseVo trafficExpenseVo) {
        //保存主体信息
        TrafficExpenseEntity trafficExpense = new TrafficExpenseEntity();
        BeanUtils.copyProperties(trafficExpenseVo, trafficExpense);
        trafficExpense.setSubject(JSON.toJSONString(trafficExpenseVo.getSubject()));
        trafficExpense.setCreateTime(new Date());
        TrafficExpenseEntity save = trafficExpenseMapper.save(trafficExpense);
        //保存到费用列表
        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("交通费");
        UserInfoEntity userInfoEntity = findUser(trafficExpenseVo.getApplicant());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(trafficExpenseVo.getProjectName());
        costInfoEntity.setPerformAmount(trafficExpenseVo.getTotalMoney());
        costInfoEntity.setProposer(trafficExpenseVo.getApplicant());
        costInfoEntity.setApplyCostId(save.getId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg trafficSubmitBySave(TrafficExpenseVo trafficExpenseVo) {
        ResMsg trafficExpenseApply = addTraffic(trafficExpenseVo);
        if (!trafficExpenseApply.getCode().equals("200")) {
            return trafficExpenseApply;
        }
        TrafficExpenseEntity save = (TrafficExpenseEntity) trafficExpenseApply.getData();
        /**
         * 调用项目交通费用申请流程的启动
         */
        ResMsg resMsg = trafficExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        trafficExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg trafficSubmitBatch(List<TrafficExpenseVo> trafficExpenseVos) {
        for (TrafficExpenseVo trafficExpenseVo : trafficExpenseVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(trafficExpenseVo.getId());
            costInfoEntity.setCostType("交通费");
            //查询交通费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //采购费用申请流程的启动
            TrafficExpenseEntity save = (TrafficExpenseEntity) infoDetails.getData();
            ResMsg resMsg = trafficExpenseFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            trafficExpenseMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg trafficUpdate(TrafficExpenseVo trafficExpenseVo) {
        //保存主体信息
        TrafficExpenseEntity trafficExpense = new TrafficExpenseEntity();
        BeanUtils.copyProperties(trafficExpenseVo, trafficExpense);
        trafficExpense.setSubject(JSON.toJSONString(trafficExpenseVo.getSubject()));
        trafficExpense.setCreateTime(new Date());
        trafficExpense.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        TrafficExpenseEntity save = trafficExpenseMapper.save(trafficExpense);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getId(), "交通费");
        UserInfoEntity userInfoEntity = findUser(trafficExpenseVo.getApplicant());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(trafficExpenseVo.getProjectName());
        costInfo.setPerformAmount(trafficExpenseVo.getTotalMoney());
        costInfo.setProposer(trafficExpenseVo.getApplicant());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg trafficSubmitByUpdate(TrafficExpenseVo trafficExpenseVo) {
        //修改后提交
        ResMsg update = trafficUpdate(trafficExpenseVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        TrafficExpenseEntity save = (TrafficExpenseEntity) update.getData();
        //调用项目交通费用申请流程的启动
        ResMsg resMsg = trafficExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        trafficExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 调用项目交通费用申请流程的启动
     * 获取开始节点之后的第一个任务节点的数据
     */
    public ResMsg trafficExpenseFlowStart(TrafficExpenseEntity trafficExpense) {
        //param processDefinitionKey 项目交通费用报销审批流程--流程标识。
        String processDefinitionKey = "projectTrafficExpenseApp";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", trafficExpense.getProjectName());
        stringObjectHashMap.put("applicant", trafficExpense.getApplicant());
        stringObjectHashMap.put("subject", trafficExpense.getSubject());
        stringObjectHashMap.put("total_money", trafficExpense.getTotalMoney());
        stringObjectHashMap.put("id", trafficExpense.getId());
        stringObjectHashMap.put("cost_type", "交通费");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(trafficExpense.getId(), "交通费");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg repastExpenseApply(RepastExpenseVo repastExpenseVo) {
        //保存主体信息
        RepastExpenseEntity repastExpense = new RepastExpenseEntity();
        BeanUtils.copyProperties(repastExpenseVo, repastExpense);
        repastExpense.setSubject(JSON.toJSONString(repastExpenseVo.getSubject()));
        repastExpense.setCreateTime(new Date());
        RepastExpenseEntity save = repastExpenseMapper.save(repastExpense);
        //保存到费用列表
        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("餐饮费");
        UserInfoEntity userInfoEntity = findUser(repastExpenseVo.getApplicant());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(repastExpenseVo.getProjectName());
        costInfoEntity.setPerformAmount(repastExpenseVo.getTotalMoney());
        costInfoEntity.setProposer(repastExpenseVo.getApplicant());
        costInfoEntity.setApplyCostId(save.getId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setCreateDate(new Date());
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg repastSubmitBySave(RepastExpenseVo repastExpenseVo) {
        ResMsg repastExpenseApply = repastExpenseApply(repastExpenseVo);
        if (!repastExpenseApply.getCode().equals("200")) {
            return repastExpenseApply;
        }
        RepastExpenseEntity save = (RepastExpenseEntity) repastExpenseApply.getData();
        /**
         * 调用项目餐饮用申请流程的启动
         */
        ResMsg resMsg = repastExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        repastExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg repastSubmitBatch(List<RepastExpenseVo> repastExpenseVos) {
        for (RepastExpenseVo repastExpenseVo : repastExpenseVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(repastExpenseVo.getId());
            costInfoEntity.setCostType("餐饮费");
            //查询餐饮费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //餐饮费用申请流程的启动
            RepastExpenseEntity save = (RepastExpenseEntity) infoDetails.getData();
            ResMsg resMsg = repastExpenseFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            repastExpenseMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg repastUpdate(RepastExpenseVo repastExpenseVo) {
        //保存主体信息
        RepastExpenseEntity repastExpense = new RepastExpenseEntity();
        BeanUtils.copyProperties(repastExpenseVo, repastExpense);
        repastExpense.setSubject(JSON.toJSONString(repastExpenseVo.getSubject()));
        repastExpense.setCreateTime(new Date());
        repastExpense.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        RepastExpenseEntity save = repastExpenseMapper.save(repastExpense);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getId(), "餐饮费");
        UserInfoEntity userInfoEntity = findUser(repastExpenseVo.getApplicant());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(repastExpenseVo.getProjectName());
        costInfo.setPerformAmount(repastExpenseVo.getTotalMoney());
        costInfo.setProposer(repastExpenseVo.getApplicant());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg repastSubmitByUpdate(RepastExpenseVo repastExpenseVo) {
        //修改后提交
        ResMsg update = repastUpdate(repastExpenseVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        RepastExpenseEntity save = (RepastExpenseEntity) update.getData();
        //调用项目餐饮用申请流程的启动
        ResMsg resMsg = repastExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        repastExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 调用项目餐饮用申请流程的启动
     * 获取开始节点之后的第一个任务节点的数据
     */
    public ResMsg repastExpenseFlowStart(RepastExpenseEntity repastExpense) {
        //* param processDefinitionKey 项目餐饮费用报销审批流程--流程标识。
        String processDefinitionKey = "projectRepastExpenseApp";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", repastExpense.getProjectName());
        stringObjectHashMap.put("applicant", repastExpense.getApplicant());
        stringObjectHashMap.put("subject", repastExpense.getSubject());
        stringObjectHashMap.put("total_money", repastExpense.getTotalMoney());
        stringObjectHashMap.put("id", repastExpense.getId());
        stringObjectHashMap.put("cost_type", "餐饮费");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(repastExpense.getId(), "餐饮费");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }


    @Override
    public ResMsg travelExpenseApply(TravelExpenseVo travelExpenseVo) {
        //保存主体信息
        TravelExpenseEntity travelExpense = new TravelExpenseEntity();
        BeanUtils.copyProperties(travelExpenseVo, travelExpense);
        travelExpense.setSubject(JSON.toJSONString(travelExpenseVo.getSubject()));
        travelExpense.setCreateTime(new Date());
        TravelExpenseEntity save = travelExpenseMapper.save(travelExpense);
        //保存到费用列表
        CostInfoEntity costInfoEntity = new CostInfoEntity();
        costInfoEntity.setCostType("差旅费");
        UserInfoEntity userInfoEntity = findUser(travelExpenseVo.getApplicant());
        costInfoEntity.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfoEntity.setProjectName(travelExpenseVo.getProjectName());
        costInfoEntity.setPerformAmount(travelExpenseVo.getTotalMoney());
        costInfoEntity.setProposer(travelExpenseVo.getApplicant());
        costInfoEntity.setApplyCostId(save.getId());
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfoEntity.setCreateDate(new Date());
        costInfoEntity.setProcessInstanceId("");
        costInfoEntity.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg travelSubmitBySave(TravelExpenseVo travelExpenseVo) {
        ResMsg travelExpenseApply = travelExpenseApply(travelExpenseVo);
        if (!travelExpenseApply.getCode().equals("200")) {
            return travelExpenseApply;
        }
        TravelExpenseEntity save = (TravelExpenseEntity) travelExpenseApply.getData();
        /**
         * 调用项目差旅费用申请流程的启动
         */
        ResMsg resMsg = travelExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        travelExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg travelSubmitBatch(List<TravelExpenseVo> travelExpenseVos) {
        for (TravelExpenseVo travelExpenseVo : travelExpenseVos) {
            CostInfoEntity costInfoEntity = new CostInfoEntity();
            costInfoEntity.setApplyCostId(travelExpenseVo.getId());
            costInfoEntity.setCostType("差旅费");
            //查询差旅费用详情
            ResMsg infoDetails = costInfoService.findCostInfoDetails(costInfoEntity);
            if (!"200".equals(infoDetails.getCode())) {
                return infoDetails;
            }
            //差旅费用申请流程的启动
            TravelExpenseEntity save = (TravelExpenseEntity) infoDetails.getData();
            ResMsg resMsg = travelExpenseFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
            save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
            travelExpenseMapper.save(save);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg travelUpdate(TravelExpenseVo travelExpenseVo) {
        //保存主体信息
        TravelExpenseEntity travelExpense = new TravelExpenseEntity();
        BeanUtils.copyProperties(travelExpenseVo, travelExpense);
        travelExpense.setSubject(JSON.toJSONString(travelExpenseVo.getSubject()));
        travelExpense.setCreateTime(new Date());
        travelExpense.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        TravelExpenseEntity save = travelExpenseMapper.save(travelExpense);
        //保存到费用列表
        CostInfoEntity costInfo = costInfoMapper.findByApplyCostIdAndCostType(save.getId(), "差旅费");
        UserInfoEntity userInfoEntity = findUser(travelExpenseVo.getApplicant());
        costInfo.setRelevanceUser(userInfoEntity != null ? userInfoEntity.getShowName() : null);
        costInfo.setProjectName(travelExpenseVo.getProjectName());
        costInfo.setPerformAmount(travelExpenseVo.getTotalMoney());
        costInfo.setProposer(travelExpenseVo.getApplicant());
        costInfo.setFlowTaskStatus(FlowTaskStatus.DRAFT);
        costInfo.setProcessInstanceId("");
        costInfo.setProcessDefinitionId("");
        costInfoService.addCostInfo(costInfo);
        return new ResMsg(CodeEnum.OK, save);
    }

    @Override
    public ResMsg travelSubmitByUpdate(TravelExpenseVo travelExpenseVo) {
        //修改后提交
        ResMsg update = travelUpdate(travelExpenseVo);
        if (!update.getCode().equals("200")) {
            return update;
        }
        TravelExpenseEntity save = (TravelExpenseEntity) update.getData();
        //调用项目差旅费用申请流程的启动
        ResMsg resMsg = travelExpenseFlowStart(save);
        if (!"200".equals(resMsg.getCode())) {
            return resMsg;
        }
        save.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        travelExpenseMapper.save(save);
        return new ResMsg(CodeEnum.OK);
    }

    public ResMsg travelExpenseFlowStart(TravelExpenseEntity travelExpense) {
        /**
         * 调用项目差旅费用申请流程的启动
         * 获取开始节点之后的第一个任务节点的数据
         * param processDefinitionKey 项目差旅费用报销审批流程--流程标识。
         */
        String processDefinitionKey = "projectTravelExpenseApp";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        //将申请信息添加到流程引擎控制的表中，并添加关联
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_name", travelExpense.getProjectName());
        stringObjectHashMap.put("applicant", travelExpense.getApplicant());
        stringObjectHashMap.put("subject", travelExpense.getSubject().toString());
        stringObjectHashMap.put("total_money", travelExpense.getTotalMoney());
        stringObjectHashMap.put("begin_time", travelExpense.getBeginTime());
        stringObjectHashMap.put("end_time", travelExpense.getEndTime());
        stringObjectHashMap.put("start_site", travelExpense.getStartSite());
        stringObjectHashMap.put("end_site", travelExpense.getEndSite());
        stringObjectHashMap.put("accommodation_fee", travelExpense.getAccommodationFee());
        stringObjectHashMap.put("accommodation_days", travelExpense.getAccommodationDays());
        stringObjectHashMap.put("total_fee", travelExpense.getTotalFee());
        stringObjectHashMap.put("accommodation_remark", travelExpense.getAccommodationRemark());
        stringObjectHashMap.put("accommodation_person", travelExpense.getAccommodationPerson());
        stringObjectHashMap.put("repast_fee", travelExpense.getRepastFee());
        stringObjectHashMap.put("repast_remark", travelExpense.getRepastRemark());
        stringObjectHashMap.put("repast_person", travelExpense.getRepastPerson());
        stringObjectHashMap.put("id", travelExpense.getId());
        stringObjectHashMap.put("cost_type", "差旅费");
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        //流程的任务变量（暂时不用）
        //HashMap<String, Object> taskVariableDataMap = new HashMap<>();
        //taskVariableDataMap.put("accommodationDays", travelExpense.getAccommodationDays());
        //JSONObject taskVariableData = (JSONObject) JSONObject.toJSON(taskVariableDataMap);
        //JSONObject copyData = null;
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        CostInfoEntity costInfoEntity = costInfoMapper.findByApplyCostIdAndCostType(travelExpense.getId(), "差旅费");
        costInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        costInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        costInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        costInfoService.addCostInfo(costInfoEntity);
        return new ResMsg(CodeEnum.OK);
    }

    /**
     * 查询用户信息
     */
    private UserInfoEntity findUser(String userId) {
        Optional<UserInfoEntity> byId = userInfoMapper.findById(userId);
        return byId.orElse(null);
    }
}
