package com.ankaibei.workFlow.webadmin.ankaibei.service.impl;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.ankaibei.workFlow.common.core.object.ResponseResult;
import com.ankaibei.workFlow.common.core.object.TokenData;
import com.ankaibei.workFlow.common.flow.constant.FlowConstant;
import com.ankaibei.workFlow.common.flow.constant.FlowTaskStatus;
import com.ankaibei.workFlow.common.flow.dto.FlowTaskCommentDto;
import com.ankaibei.workFlow.common.flow.model.FlowEntry;
import com.ankaibei.workFlow.common.flow.model.FlowEntryPublish;
import com.ankaibei.workFlow.common.flow.vo.TaskInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.ManHourInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.ProjectInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.TaskInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.entity.UserInfoEntity;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.CodeEnum;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.DataIsDeleteEnum;
import com.ankaibei.workFlow.webadmin.ankaibei.enums.ManHourStatus;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.ManHourMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.ProjectInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.TaskInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.mapper.UserInfoMapper;
import com.ankaibei.workFlow.webadmin.ankaibei.service.ManHourService;
import com.ankaibei.workFlow.webadmin.ankaibei.utils.FlowTaskStart;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ManHourInfoVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ManHourJobVo;
import com.ankaibei.workFlow.webadmin.ankaibei.vo.ResMsg;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @author lwq
 * @version 1.0
 * @description:
 * @date 2022/9/21 15:33
 */
@Service
public class ManHourServiceImpl implements ManHourService {

    @Resource
    private ManHourMapper manHourMapper;
    @Resource
    private FlowTaskStart flowTaskStart;
    @Resource
    private UserInfoMapper userInfoMapper;
    @Resource
    private ProjectInfoMapper projectInfoMapper;
    @Resource
    private TaskInfoMapper taskInfoMapper;

    @Override
    public ResMsg findManHourAll(Integer pageIndex, Integer pageSize, ManHourInfoEntity manHourInfoEntity) {
        //拿到用户的ID，只查询与本人有关的工时
        String userId = TokenData.takeFromRequest().getUserId();
        Pageable page = PageRequest.of(pageIndex - 1, pageSize);
        String endDate = null;
        if (manHourInfoEntity.getEndDate() != null) {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
            endDate = dateFormat.format(manHourInfoEntity.getEndDate()) + " 23:59:59";
        }
        Page<Map<String, String>> manHourAll = manHourMapper.findManHourAll(userId, manHourInfoEntity.getFlowTaskStatus(), manHourInfoEntity.getStartDate(), endDate, page);
        //匹配项目信息和任务信息
        List<ProjectInfoEntity> projectInfos = projectInfoMapper.findAll();
        List<TaskInfoEntity> finishedTasks = taskInfoMapper.findAllByTaskParticipantContainsAndTaskStatusEqualsAndIsDeleteEquals(userId, String.valueOf(FlowTaskStatus.FINISHED), (DataIsDeleteEnum.NORMAL.getCode()));
        return new ResMsg(CodeEnum.OK, ResMsg.getMap(manHourAll.map(map -> {
            Map<Object, Object> map1 = new HashMap<>(map);
            //匹配任务信息
            for (TaskInfoEntity taskInfo : finishedTasks) {
                if (map1.get("taskId").equals(taskInfo.getTaskId().toString())) {
                    map1.put("targetTask", taskInfo.getTargetTask());
                    break;
                }
            }
            String projectId = map.get("projectId");
            for (ProjectInfoEntity projectInfo : projectInfos) {
                if (Integer.valueOf(projectId) == (projectInfo.getProjectId())) {
                    map1.put("projectName", projectInfo.getProjectName());
                    break;
                }
            }
            return map1;
        }).getContent(), manHourAll.getTotalElements()));
    }

    @Override
    public ResMsg addManHour(ManHourInfoVo manHourInfoVo) {
        List<ManHourJobVo> jobs = manHourInfoVo.getJobs();
        Optional<UserInfoEntity> byId1 = userInfoMapper.findById(manHourInfoVo.getCreateUser());
        String showName = byId1.orElse(null) != null ? byId1.orElse(null).getShowName() : null;
        for (ManHourJobVo job : jobs) {
            ManHourInfoEntity manHourInfoEntity = new ManHourInfoEntity();
            BeanUtils.copyProperties(manHourInfoVo, manHourInfoEntity);
            //工时：添加后状态为1，未提交
            manHourInfoEntity.setProjectUser(showName);
            manHourInfoEntity.setCreateUser(byId1.get().getUserId());
            manHourInfoEntity.setManStatus(ManHourStatus.NO_ALLOCATION.getCode());
            manHourInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
            BeanUtils.copyProperties(job, manHourInfoEntity);
            manHourInfoEntity.setCreateTime(new Date());
            manHourInfoEntity.setIsDelete(DataIsDeleteEnum.NORMAL.getCode());
            manHourMapper.save(manHourInfoEntity);
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg submitManHourBySave(ManHourInfoVo manHourInfoVo) {
        List<ManHourJobVo> jobs = manHourInfoVo.getJobs();
        Optional<UserInfoEntity> byId1 = userInfoMapper.findById(manHourInfoVo.getCreateUser());
        String showName = byId1.orElse(null) != null ? byId1.orElse(null).getShowName() : null;
        for (ManHourJobVo job : jobs) {
            ManHourInfoEntity manHourInfoEntity = new ManHourInfoEntity();
            BeanUtils.copyProperties(manHourInfoVo, manHourInfoEntity);
            //工时：添加后状态为1，未提交
            manHourInfoEntity.setProjectUser(showName);
            manHourInfoEntity.setCreateUser(byId1.get().getUserId());
            manHourInfoEntity.setManStatus(ManHourStatus.NO_ALLOCATION.getCode());
            manHourInfoEntity.setFlowTaskStatus(FlowTaskStatus.DRAFT);
            BeanUtils.copyProperties(job, manHourInfoEntity);
            manHourInfoEntity.setCreateTime(new Date());
            manHourInfoEntity.setIsDelete(DataIsDeleteEnum.NORMAL.getCode());
            ManHourInfoEntity save = manHourMapper.save(manHourInfoEntity);
            ResMsg resMsg = manHourFlowStart(save);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg submitManHourBatch(List<ManHourInfoVo> manHourInfoVos) {
        //先判断ID是否为空：不为空就是批量提交的
        if (!ObjectUtils.isEmpty(manHourInfoVos)) {
            for (ManHourInfoVo vo : manHourInfoVos) {
                Optional<ManHourInfoEntity> byId = manHourMapper.findById(vo.getManId());
                ManHourInfoEntity infoEntity = byId.orElse(null) != null ? byId.orElse(null) : null;
                ResMsg resMsg = manHourFlowStart(infoEntity);
                if (!"200".equals(resMsg.getCode())) {
                    return resMsg;
                }
            }
        }
        return new ResMsg(CodeEnum.OK);
    }

    @Override
    public ResMsg deleteManHour(List<ManHourInfoVo> manHourInfoVos) {
        //审批状态   已提交：0; 审批中：1; 被拒绝：2; 已结束：3; 提前停止：4; 已取消：5;保存草稿:6
        StringBuffer optional = new StringBuffer("工时：[");
        int count = 0;
        for (ManHourInfoVo info : manHourInfoVos) {
            ManHourInfoEntity byId = manHourMapper.getById(info.getManId());
//            //如果流程状态大于1，便是不在审核中，可删除
            if (byId.getFlowTaskStatus() > 1) {
                manHourMapper.deleteById(Integer.valueOf(info.getManId()));
                count++;
            } else {
                optional.append("【" + byId.getJobContent() + "】,");
            }
        }
        optional.replace(optional.length() - 1, optional.length(), "");
        optional.append("] 审批未结束，不可删除！");
        if (count == manHourInfoVos.size()) {
            return new ResMsg(CodeEnum.OK, "");
        } else {
            return new ResMsg(CodeEnum.FAIL, optional);
        }
    }

    @Override
    public ResMsg findManHourByID(Integer manId) {
        ManHourInfoEntity byId = manHourMapper.findById(manId).get();
        //数据赋值给VO对象，返回VO对象
        ManHourInfoVo vo = new ManHourInfoVo();
        BeanUtils.copyProperties(byId, vo);
        //获取关联数据的信息，给前端展示
        Optional<UserInfoEntity> userInfoEntity = userInfoMapper.findById(byId.getCreateUser());
        //创建用户和申请人
        vo.setCreateUserName(userInfoEntity.get().getShowName());
        vo.setProjectUserName(userInfoEntity.get().getShowName());
        Optional<ProjectInfoEntity> projectInfoEntity = projectInfoMapper.findById(Integer.valueOf(byId.getProjectId()));
        //项目名称
        vo.setProjectName(projectInfoEntity.get().getProjectName());
        Optional<TaskInfoEntity> taskInfoEntity = taskInfoMapper.findById(Integer.valueOf(byId.getTaskId()));
        //任务名称
        vo.setTargetTask(taskInfoEntity.get().getTargetTask());
        return new ResMsg(CodeEnum.OK, vo);
    }

    @Override
    public ResMsg editManHour(ManHourInfoVo manHourInfoVo) {
        ManHourInfoEntity byId = manHourMapper.findById(manHourInfoVo.getManId()).get();
        String manStatus = byId.getManStatus();
        //未提交的工时更改
        if (ManHourStatus.NO_ALLOCATION.getCode().equals(manStatus)) {
            ManHourInfoEntity infoEntity = new ManHourInfoEntity();
            BeanUtils.copyProperties(manHourInfoVo, infoEntity);
            //保留原本信息
            infoEntity.setProjectUser(byId.getProjectUser());
            infoEntity.setCreateUser(byId.getCreateUser());
            infoEntity.setManStatus(ManHourStatus.NO_ALLOCATION.getCode());
            infoEntity.setCreateTime(new Date());
            infoEntity.setIsDelete(DataIsDeleteEnum.NORMAL.getCode());
            ManHourInfoEntity save = manHourMapper.save(infoEntity);
            return new ResMsg(CodeEnum.OK, save);
        } else if (ManHourStatus.REFUSE_ED.getCode().equals(manStatus)) {
            //拒绝后的工时更改
            ManHourInfoEntity infoEntity = new ManHourInfoEntity();
            BeanUtils.copyProperties(manHourInfoVo, infoEntity);
            //保留
            infoEntity.setProjectUser(byId.getProjectUser());
            infoEntity.setCreateUser(byId.getCreateUser());
            infoEntity.setManStatus(ManHourStatus.NO_ALLOCATION.getCode());
            infoEntity.setCreateTime(new Date());
            infoEntity.setIsDelete(DataIsDeleteEnum.NORMAL.getCode());
            infoEntity.setManId(new ManHourInfoEntity().getManId());
            manHourMapper.deleteById(byId.getManId());
            ManHourInfoEntity save = manHourMapper.save(infoEntity);
            return new ResMsg(CodeEnum.OK, save);
        }
        return new ResMsg(CodeEnum.OK, null);
    }

    /**
     * 编辑工时之后提交
     *
     * @param manHourInfoVo
     * @return
     */
    @Override
    public ResMsg submitManHourByeEdit(ManHourInfoVo manHourInfoVo) {
        //先编辑工时
        ManHourInfoEntity infoEntity = (ManHourInfoEntity) editManHour(manHourInfoVo).getData();
        // 提交工时
        if (!ObjectUtils.isEmpty(infoEntity)) {
            ResMsg resMsg = manHourFlowStart(infoEntity);
            if (!"200".equals(resMsg.getCode())) {
                return resMsg;
            }
        } else {
            return new ResMsg(CodeEnum.DATA_VALIDATED_FAILED);
        }
        return new ResMsg(CodeEnum.OK);
    }

    public ResMsg manHourFlowStart(ManHourInfoEntity manHourInfoEntity) {
        //param processDefinitionKey 项目人员工时接口审批流程--流程标识。
        String processDefinitionKey = "manHourCountApply";
        ResponseResult<FlowEntry> flowEntryResult = flowTaskStart.verifyAndGetFlowEntry(processDefinitionKey);
        if (!flowEntryResult.isSuccess()) {
            return ResMsg.newInstance(CodeEnum.FAIL, flowEntryResult);
        }
        //调用人员工时申请流程的启动
        //获取流程开始节点之后的第一个任务节点的数据
        FlowEntryPublish flowEntryPublish = flowEntryResult.getData().getMainFlowEntryPublish();
        String initTaskInfo = flowEntryPublish.getInitTaskInfo();
        com.ankaibei.workFlow.common.flow.vo.TaskInfoVo taskInfo = StrUtil.isBlank(initTaskInfo)
                ? null : JSON.parseObject(initTaskInfo, TaskInfoVo.class);
        if (taskInfo != null) {
            String loginName = TokenData.takeFromRequest().getLoginName();
            taskInfo.setAssignedMe(StrUtil.equalsAny(
                    taskInfo.getAssignee(), loginName, FlowConstant.START_USER_NAME_VAR));
        }
        //用户在表单页面配置的操作按钮
        FlowTaskCommentDto flowTaskCommentDto = new FlowTaskCommentDto();
        //提交后进行到第二个节点
        flowTaskCommentDto.setApprovalType("agree");
        HashMap<String, Object> stringObjectHashMap = new HashMap<>();
        stringObjectHashMap.put("project_user", manHourInfoEntity.getProjectUser());
        stringObjectHashMap.put("project_name", manHourInfoEntity.getProjectId());
        stringObjectHashMap.put("man_hour", "总计**时");
        stringObjectHashMap.put("create_date", manHourInfoEntity.getStartDate());
        stringObjectHashMap.put("create_user", manHourInfoEntity.getCreateUser());
        ManHourInfoEntity save = manHourMapper.save(manHourInfoEntity);
        stringObjectHashMap.put("man_id", save.getManId());
        JSONObject masterData = (JSONObject) JSONObject.toJSON(stringObjectHashMap);
        Map<String, String> map = flowTaskStart.startAndTakeUserTaskSS(processDefinitionKey, flowTaskCommentDto, null, masterData, null, null).getData();
        manHourInfoEntity.setProcessInstanceId(map.get("processInstanceId"));
        manHourInfoEntity.setProcessDefinitionId(map.get("processDefinitionId"));
        manHourInfoEntity.setManStatus(ManHourStatus.EXTENSION_ING.getCode());
        manHourInfoEntity.setFlowTaskStatus(FlowTaskStatus.SUBMITTED);
        manHourInfoEntity.setCreateTime(new Date());
        ManHourInfoEntity save1 = manHourMapper.save(manHourInfoEntity);
        return new ResMsg(CodeEnum.OK, save1);
    }


}
